import 'package:dio/dio.dart';
import 'package:sw_auth/controller_polled_auth.dart';
import 'package:sw_auth/endpoint/endpoint_polled_auth.dart';
import 'package:sw_auth/service/service_token.dart';
import 'package:sw_core/environment/sw_env.dart';
import 'package:sw_core/interface/controller/i_controller_auth.dart';
import 'package:sw_core/interface/controller/i_controller_polled_auth.dart';
import 'package:sw_core/interface/service/i_service_token.dart';
import 'package:sw_core/request/request_auth_token.dart';
import 'package:sw_core/sw_init_core.dart';
import 'package:sw_core/tool/data_path.dart';
import 'package:sw_core/tool/http_client.dart';
import 'package:sw_core/tool/inject.dart';
import 'package:sw_auth/endpoint/endpoint_auth_token.dart';
import 'package:sw_core/tool/log.dart';
import 'controller_auth.dart';
import 'endpoint/endpoint_auth.dart';
import 'endpoint/endpoint_auth_refresh_token.dart';

void initAuth({RequestAuthToken? requestToken, String? token}) {
  initCore();
  _initToken(requestToken, token);
  _initControllerPolledAuth();
  _initController();
}

void _initToken(RequestAuthToken? requestToken, String? token) {
  if (requestToken != null) {
    wtf("Initializing using partner provided token endpoint!");
    _initServiceWithRequest(requestToken);
    return;
  }
  if (token != null) {
    wtf("Initializing using partner provided token string!");
    _initServiceWithToken(token);
    return;
  }
  error("No token or token endpoint was provided, token service couldn't be initialized!");
}

void _initServiceWithRequest(RequestAuthToken requestToken) => put<IServiceToken>(
      () => ServiceToken(
        endToken: EndpointAuthToken(defaultHTTPClient(), baseUrl: requestToken.endpoint),
        endRefreshToken: EndpointAuthRefreshToken(defaultHTTPClient()),
        requestToken: requestToken,
      ),
    );

void _initServiceWithToken(String token) => put<IServiceToken>(() => ServiceToken(token: token));

Dio _getDioAdapter() {
  Dio _adapter = get<Dio>();
  // _adapter.options = BaseOptions(validateStatus: _validateStatus);
  return _adapter;
}

void _initControllerPolledAuth() => put<IControllerPolledAuth>(
      () => ControllerPolledAuth(
        EndpointPolledAuth(
          _getDioAdapter(),
          baseUrl: get<SWEnvironment>() == SWEnvironment.DEV ? base_SW_URL_SANDBOX : base_SW_URL,
        ),
      ),
    );

// bool _validateStatus(int? status) => status != null ? status <= 304 : false;

void _initController() => put<IControllerAuth>(
      () => ControllerAuth(
        EndpointAuth(
          get<Dio>(),
          baseUrl: get<SWEnvironment>() == SWEnvironment.DEV ? base_SW_URL_SANDBOX : base_SW_URL,
        ),
        get<IControllerPolledAuth>(),
      ),
    );
